
#include <X11/Xlib.h>
#include "DispRec/DispRecCtrlExtension.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>

#define  COMPRESS

void waitkey()
{
    getchar();
}


int main(int argc, char* argv[])
{
    Display* dpy;
    Bool haveDispRecCtrl=False;
    int event_base, error_base;
    int major_version, minor_version;
    Status status;
    int childpid;
    DispRecCtrl_RecordingSession recordingSession;

    char recordingDir[1024];
    
    sprintf( recordingDir, "/var/tmp/displayrecorder.%d", getpid() );
    mkdir(recordingDir, 0777);
    
    printf("recording dir=\"%s\"\n", recordingDir);

    dpy=XOpenDisplay(NULL);

    
    haveDispRecCtrl = 
	XDispRecCtrl_QueryExtension(
    	dpy, &event_base, &error_base);

    printf("haveDispRecCtrl=%d\n", haveDispRecCtrl);
    if (haveDispRecCtrl) {

	recordingSession=XDispRecCtrl_CreateRecordingSession( dpy );
    
	childpid = fork();
	if (childpid != 0) {
	    /***************************/
	    /* parent process */
	    /***************************/
    
	    status = XDispRecCtrl_QueryVersion(
		dpy, &major_version, &minor_version);
	    printf("Version=(%d, %d)\n", major_version, minor_version);
	    fflush(stdout);
    
    #ifdef COMPRESS
	    /* Turn on compression with gzip */
	    XDispRecCtrl_EnableCompression(dpy, recordingSession, "gzip", "/usr/bin/gzip  --fast  ");
    #else
	    XDispRecCtrl_DisableCompression(dpy, recordingSession);
    #endif
    
	    sleep(1); /* give the child process time to startup */
    
	    printf("sending request to start recording\n");
	    fflush(stdout);
    
	    XDispRecCtrl_StartRecording(
		dpy, recordingSession, recordingDir, 0, 
		/* endless = */ False, 
		/* doSoundRecording = */ False, 
		/* muteSoundRecording = */ False
	    );
	    XFlush(dpy);
	    
	    /* wait some time, so display of the returned events does not interfere with display
		of the "press key to finish recording" */
	    sleep(1); 
    
	    printf ("press key to finish recording");
	    fflush(stdout);
	    waitkey();
    
	    printf("sending request to finish recording\n");
	    fflush(stdout);
	    XDispRecCtrl_FinishRecording(dpy, recordingSession);
	    XFlush(dpy);
    
	    /* wait for child process to terminate */
	    {
		int status;
		waitpid( childpid, &status, 0 );
	    }
	    XFlush(dpy);
	    XCloseDisplay(dpy);
	    return 0;
	} else {
	    /***************************/
	    /* Child process (listen for events) */
	    /***************************/
	    dpy=XOpenDisplay(NULL);
    
	    haveDispRecCtrl = 
		XDispRecCtrl_QueryExtension(
		    dpy, &event_base, &error_base);
    
	    printf("haveDispRecCtrl=%d\n", haveDispRecCtrl);
	    if (haveDispRecCtrl) {
		/* Select events */
		XDispRecCtrl_SelectInput(dpy, DispRecCtrl_NotifyMask);
		/* Loop for events */
		Bool finished = False;
		while(!finished) {
		    XEvent event;
		    XNextEvent(dpy, &event);
		    if (event.type == event_base) {
			XDispRecCtrl_NotifyEvent *notifyevent=
			    (XDispRecCtrl_NotifyEvent*)&event;
			switch(notifyevent->subtype) {
			    case DispRecCtrl_NotifyStarted: {
				XDispRecCtrl_NotifyStartedEvent *myevent=
				    (XDispRecCtrl_NotifyStartedEvent*)notifyevent;
				printf(
				    "\nNotifyStartedEvent: recordingSession = %d\n", 
				    (int)recordingSession 
				);
			    } break;
			    case DispRecCtrl_NotifyNotStarted: {
				XDispRecCtrl_NotifyNotStartedEvent *myevent=
				    (XDispRecCtrl_NotifyNotStartedEvent*)notifyevent;
				printf(
				    "\nNotifyNotStartedEvent: recordingSession = %d, reason = %d\n", 
				    (int)recordingSession,
				    myevent->reason
				);
			    } break;
			    case DispRecCtrl_NotifyFinished: {
				XDispRecCtrl_NotifyFinishedEvent *myevent=
				    (XDispRecCtrl_NotifyFinishedEvent*)notifyevent;
				printf(
				    "\nNotifyFinishedEvent: recordingSession = %d, reason = %d\n", 
				    (int)recordingSession,
				    myevent->reason
				);
				finished = True;
			    } break;
			    case DispRecCtrl_NotifyNotFinished: {
				XDispRecCtrl_NotifyNotFinishedEvent *myevent=
				    (XDispRecCtrl_NotifyNotFinishedEvent*)notifyevent;
				printf(
				    "\nNotifyNotFinishedEvent: recordingSession = %d, reason = %d\n", 
				    (int)recordingSession,
				    myevent->reason
				);
				finished = True;
			    } break;
			}
		    }
		}
	    }	
	    return 0;
	}
    }
}

