/****************************************
Filename: error.h
Description: Error handling
Author: Christian Linhart
Copyright: (C)1997 Christian Linhart
****************************************/

#ifndef VT_error_h
#define VT_error_h

#ifndef VT_error_no_DISPREC_PRIVATE
#endif

#include <X11/Xfuncproto.h>

/* forward decl of VtErrorRec */
typedef struct VtErrorRec_struct VtErrorRec;

/*** data type for errors (pointer to error record).
  NULL means "no error" ***/
typedef VtErrorRec *VtError;


_XFUNCPROTOBEGIN

/*** Error types ***/
/* some comments for the error-types are taken from Linux
   man-pages (or from the SunOS man-pages for error-codes
   which are not described in the Linux-manpages) */
typedef enum{
    VtError_NoError
	,VtError_L1WriteError
	,VtError_L1ReadError

	/***********************************/
	/*** errors on opening a file for reading ***/
	/***********************************/

	/* no more detail on reason available */
	,VtError_CantOpenFileForReading 

	/* The requested access to the file is not allowed, or
	   one of the directories in pathname  did  not  allow
	   search (execute) permission. */
	,VtError_CantOpenFileForReading_EACCES

	/* A signal was caught  during  the  open() system call. */
	,VtError_CantOpenFileForReading_EINTR

	/* An I/O error occurred while reading from
	   or writing to the file system. */
	,VtError_CantOpenFileForReading_EIO
	
	    /* pathname contains a reference to  a  circular  sym
              bolic link, ie a symbolic link whose expansion con
              tains a reference to itself. */
	,VtError_CantOpenFileForReading_ELOOP

	    /* The process already has the maximum number of files
              open. */
	,VtError_CantOpenFileForReading_EMFILE

	    /* pathname was too long */
	,VtError_CantOpenFileForReading_ENAMETOOLONG

	    /* The limit on the total number of files open on  the
	       system has been reached. */
	,VtError_CantOpenFileForReading_ENFILE

	    /* A directory component in pathname does not exist or
              is a dangling symbolic link */
	,VtError_CantOpenFileForReading_ENOENT

	    /* A component used as a directory in pathname is not,
	       in fact, a directory. */
	,VtError_CantOpenFileForReading_ENOTDIR
	
	/****************************************/
	/*** errors on opening a  file for writing and/or ***/
	/*** on creating a file                                               ***/
	/****************************************/
	/* no more detail on reason available */
	,VtError_CantOpenFileForWriting

	/* The requested access to the file is not allowed, or
              one of the directories in pathname  did  not  allow
              search (execute) permission */
	,VtError_CantOpenFileForWriting_EACCES

	/* The file  does  not  exist,  O_CREAT  is
                         specified,  and  the  directory in which
                         the entry for  the  new  file  is  being
                         placed  cannot  be  extended because the
                         user's quota of disk blocks on the  file
                         system containing the directory has been
                         exhausted.

                         The file  does  not  exist,  O_CREAT  is
                         specified,   and  the  user's  quota  of
                         inodes on the file system on  which  the
                         file   is   being   created   has   been
                         exhausted. */
	,VtError_CantOpenFileForWriting_EDQUOT

	/* pathname already exists and O_CREAT and O_EXCL were
              used. */
	,VtError_CantOpenFileForWriting_EEXIST

	/* A signal was caught  during  the  open() system call. */
	,VtError_CantOpenFileForWriting_EINTR

	/* An I/O error occurred while reading from
	   or writing to the file system. */
	,VtError_CantOpenFileForWriting_EIO
	
	/* pathname  refers  to  a  directory  and  the access
              requested involved writing */
	,VtError_CantOpenFileForWriting_EISDIR

	/* pathname contains a reference to  a  circular  sym
              bolic link, ie a symbolic link whose expansion con
              tains a reference to itself */
	,VtError_CantOpenFileForWriting_ELOOP

	/* The process already has the maximum number of files
              open */
	,VtError_CantOpenFileForWriting_EMFILE

	/* pathname was too long */
	,VtError_CantOpenFileForWriting_ENAMETOOLONG

	/* The limit on the total number of files open on  the
              system has been reached */
	,VtError_CantOpenFileForWriting_ENFILE

	/* A directory component in pathname does not exist or
              is a dangling symbolic link */
	,VtError_CantOpenFileForWriting_ENOENT

	/* pathname was to be created but the device  contain
              ing pathname has no room for the new file */
	,VtError_CantOpenFileForWriting_ENOSPC

	/* A component used as a directory in pathname is not,
	   in fact, a directory */
	,VtError_CantOpenFileForWriting_ENOTDIR

	/* O_NDELAY is set, the  named  file  is  a
                         FIFO,  O_WRONLY  is  set, and no process
                         has the file open for reading */
	,VtError_CantOpenFileForWriting_ENXIO

	/* pathname refers to a file on a read-only filesystem
              and write access was requested */
	,VtError_CantOpenFileForWriting_EROFS

	/* pathname refers to an  executable  image  which  is
              currently  being  executed  and  write  access  was
              requested. */
	,VtError_CantOpenFileForWriting_ETXTBSY

	/****************************************/
	/*** errors on closing a file for writing ***/
	/****************************************/

	/* no more detail on reason available */
	,VtError_CantCloseFileForWriting

	/* fd isn't a valid open file descriptor. */
	,VtError_CantCloseFileForWriting_EBADF

	/* An I/O error occurred. */
	,VtError_CantCloseFileForWriting_EIO

	/***********************************/
	/*** errors on opening a file for reading ***/
	/***********************************/
	/* no more detail on reason available */
	,VtError_CantOpenDir
	
	/* Search permission is denied for the component  of  the
	path  prefix  of  dirname or read permission is denied
	for dirname. */
	,VtError_CantOpenDir_EACCESS
	
	/* Too many symbolic links were encountered in  resolving
	path. */
	,VtError_CantOpenDir_ELOOP
	
	/* The length of the dirname argument  exceeds  PATH_MAX,
       or  a pathname component is longer than NAME_MAX while
       _POSIX_NO_TRUNC is in effect. 
	    or
	Pathname resolution of a  symbolic  link  produced  an
	intermediate result whose length exceeds PATH_MAX. */
	,VtError_CantOpenDir_ENAMETOOLONG

	/* A component of  dirname  does  not  name  an  existing
	directory or dirname is an empty string. */
	,VtError_CantOpenDir_ENOENT

	/* A component of dirname is not a directory. */
	,VtError_CantOpenDir_ENOTDIR

	/* There are OPEN_MAX file descriptors currently open  in
	the calling process. */
	,VtError_CantOpenDir_EMFILE

	/* Too many files are currently open in the system. */
	,VtError_CantOpenDir_ENFILE
	
	/* the directory does not contain a keyfile, thus it does not contain a valid recording */
	, VtError_CantOpenDir_NoKeyFile

	/****************************************/
	/*** errors on closing a file for reading ***/
	/****************************************/

	/* no more detail on reason available */
	,VtError_CantCloseFileForReading

	/* fd isn't a valid open file descriptor. */
	,VtError_CantCloseFileForReading_EBADF

	/* An I/O error occurred. */
	,VtError_CantCloseFileForReading_EIO

	/****************************/
	/*** errors with out of memory ***/
	/****************************/
	,VtError_OutOfMemory
	,VtError_BaseServerOutOfMemory
	,VtError_RpservOutOfMemory
} VtErrorType;


/*** Data records for error types ***/



/* for 
   VtError_CantOpenFileForReading,
   VtError_CantOpenFileForReading_...,
   VtError_CantOpenFileForWriting,
   VtError_CantOpenFileForWriting_...,
   */
typedef struct{
    char* filename;
} VtError_FileOpenError;

typedef struct{
    int fd;
    char* filename;
} VtError_FileCloseError;


/* for
    VtError_CantOpenDir,
    VtError_CantOpenDir_...
    */
typedef struct{
    char* dirname;
} VtError_DirOpenError;

/*** error record ***/
struct VtErrorRec_struct{
    VtErrorType type;
    Bool free_on_delete; /* True: May be freed,
			  False: is statically allocated --> may not be freed */
    union{
	VtError_FileOpenError file_open;
	VtError_FileCloseError file_close;
	VtError_DirOpenError dir_open;
    } data;
};


/***********************************/
/* Macros for convenient handling of errors */
/***********************************/

/* Some classes of errors (for use in switch statements) */
#define VtError_CantOpenFileForReading_CLASS \
VtError_CantOpenFileForReading: \
case VtError_CantOpenFileForReading_EACCES: \
case VtError_CantOpenFileForReading_EINTR: \
case VtError_CantOpenFileForReading_EIO: \
case VtError_CantOpenFileForReading_ELOOP: \
case VtError_CantOpenFileForReading_EMFILE: \
case VtError_CantOpenFileForReading_ENAMETOOLONG: \
case VtError_CantOpenFileForReading_ENFILE: \
case VtError_CantOpenFileForReading_ENOENT: \
case VtError_CantOpenFileForReading_ENOTDIR
	
#define VtError_CantOpenFileForWriting_CLASS \
VtError_CantOpenFileForWriting: \
case VtError_CantOpenFileForWriting_EACCES: \
case VtError_CantOpenFileForWriting_EDQUOT: \
case VtError_CantOpenFileForWriting_EEXIST: \
case VtError_CantOpenFileForWriting_EINTR: \
case VtError_CantOpenFileForWriting_EIO: \
case VtError_CantOpenFileForWriting_EISDIR: \
case VtError_CantOpenFileForWriting_ELOOP: \
case VtError_CantOpenFileForWriting_EMFILE: \
case VtError_CantOpenFileForWriting_ENAMETOOLONG: \
case VtError_CantOpenFileForWriting_ENFILE: \
case VtError_CantOpenFileForWriting_ENOENT: \
case VtError_CantOpenFileForWriting_ENOSPC: \
case VtError_CantOpenFileForWriting_ENOTDIR: \
case VtError_CantOpenFileForWriting_ENXIO: \
case VtError_CantOpenFileForWriting_EROFS: \
case VtError_CantOpenFileForWriting_ETXTBSY

#define VtError_CantOpenDir_CLASS \
VtError_CantOpenDir: \
case VtError_CantOpenDir_EACCESS: \
case VtError_CantOpenDir_ELOOP: \
case VtError_CantOpenDir_ENAMETOOLONG: \
case VtError_CantOpenDir_ENOENT: \
case VtError_CantOpenDir_ENOTDIR: \
case VtError_CantOpenDir_EMFILE: \
case VtError_CantOpenDir_ENFILE: \
case VtError_CantOpenDir_NoKeyFile


/* Declare an err-pointer and init it with no-error */
#define DECLERR \
VtError errval = NULL; \
VtError *err = &errval \

/* Delete the current error */
#define DELERR if ( (*err != NULL) && (*err)->free_on_delete ) { \
			   VtError_DeleteErrorData(*err); \
			   DELETE(*err); } \
                           *err = NULL; 

#define ERROR_TYPE() ((*err)->type)

#define ERROR_TYPE_ARG(errorObject) ((errorObject)->type)

#define HAVE_ERROR() ((*err != NULL) && ( ERROR_TYPE() != VtError_NoError ) )

#define HAVE_ERROR_ARG(errorObject) \
    ((errorObject != NULL) && ( ERROR_TYPE_ARG(errorObject) != VtError_NoError ) )

/* Return a value on error */
#define RETVALONERR \
if (HAVE_ERROR()) return ERR_RETVAL

/* Return on error in a void function */
#define RETONERR \
if (HAVE_ERROR()) return

/* Return a value on error and delete the error */
#define DEL_RETVALONERR \
if (HAVE_ERROR()) { \
    DELERR; \
    return ERR_RETVAL; \
}

/* Return on error in a void function */
#define DEL_RETONERR \
if (HAVE_ERROR()) { \
    DELERR; \
    return; \
}

#define ERROR_DATA(kind) ((*err)->data.kind)

#define NEW_ERROR  VtError_New(err)


/*********************************************/
/* Functions which may only be used by the previously */
/* defined macros                                                                       */
/*********************************************/

void VtError_DeleteErrorData(VtError err);

void VtError_New(VtError *err);

/* generates a string description of the given error. 
    The string has to be deallocated by the caller using DELETEARRAY */
char* VtErrorToString( VtError errorObject );

_XFUNCPROTOEND

#endif
